/*
 * Oppgave21_1.java ST/EL 2001-06-15
 *
 * Denne oppgaven omfatter at man skal utvide programliste 21.4 til  ogs
 * sttte sletting av data fra sketreet. Dette er en ganske omfattende
 * operasjon, men ved  implementere de ndvendige funksjonene beskrevet i
 *
 *   "Introduction to Algorithms" av Cormen, Leiserson og Rivest
 *
 * blir oppgaven betraktelig enklere. Under kapittel 13, "Binary Search Trees",
 * finnes pseudokoden for bde innsetting og sletting av data.
 */
public class Oppgave21_1 {
  public static void main(String[] args) {

    /* Fyller et tre med tilfeldig data. */
    SubTre tre = new SubTre(6);
    for (int i = 1; i < 25; i++) {
      int verdi = 0;
      do {
        verdi = (int)(Math.random() * 50);
      }
      while (tre.skEtterVerdi(verdi));

      tre.settInnVerdi(verdi);
    }

    System.out.println("Treet: " + tre + "\n");

    /* Prver  slette en rekke tilfeldige tall. */
    for (int i = 0; i < 5; i++) {
      int verdi = 0;
      do {
        verdi = (int)(Math.random() * 50);
      }
      while (!tre.skEtterVerdi(verdi));

      slettVerdi(tre, verdi);
      System.out.println("Slettet tallet " + verdi + ".\n" +
                          "Treet: " + tre + "\n");
    }
  }

  /*
   * Metoden sletter en gitt verdi fra det gitte treet.
   */
  public static boolean slettVerdi(SubTre tre, int enVerdi) {
    SubTre z = tre.finnSubTre(enVerdi);
    if (z == null)
      return false;

    SubTre y = null;
    if (z.finnVenstreTre() == null || z.finnHyreTre() == null)
      y = z;
    else
      y = z.finnEtterflger();

    SubTre x = null;
    if (y.finnVenstreTre() != null)
      x = y.finnVenstreTre();
    else
      x = y.finnHyreTre();

    if (x != null)
      x.settForelder(y.finnForelder());

    if (y.finnForelder() == null)
      tre = x;
    else if (y == y.finnForelder().finnVenstreTre())
      y.finnForelder().settVenstreTre(x);
    else
      y.finnForelder().settHyreTre(x);

    if (y != z)
      z.settVerdi(y.finnVerdi());

    return true;
  }
}

/*
 * Klasse for delene av et binrt sketre. Subtreene, inklusive
 * bladene, bestr av nye objekter av klassen.
 */
class SubTre {
  private SubTre hyreTre = null;
  private SubTre venstreTre = null;
  private SubTre forelder = null;
  private int verdi = 0;

  public SubTre(int startVerdi) {
    verdi = startVerdi;
  }

  public SubTre(int startVerdi, SubTre startForelder) {
    verdi = startVerdi;
    forelder = startForelder;
  }

  /*
   * Metoden setter inn en ny verdi i dette (sub)treet.
   */
  public void settInnVerdi(int nyVerdi) {

    /* Dersom verdien for dette hjrnet er strre enn eller likden
     * nye, skal den nye verdien inn i venstre subtre. */
    if (verdi >= nyVerdi) {
      if (venstreTre != null) {
        venstreTre.settInnVerdi(nyVerdi);
      } else {
        venstreTre = new SubTre(nyVerdi, this);
      }
    } else {
      if (hyreTre != null) {
        hyreTre.settInnVerdi(nyVerdi);
      } else {
        hyreTre = new SubTre(nyVerdi, this);
      }
    }
  }

  /*
   * Finn-metoder
   */
  public SubTre finnVenstreTre() {
    return venstreTre;
  }

  public SubTre finnHyreTre() {
    return hyreTre;
  }

  public SubTre finnForelder() {
    return forelder;
  }

  public SubTre finnRot() {
    if (forelder == null)
      return this;
    else
      return forelder.finnRot();
  }

  public SubTre finnMin() {
    if (venstreTre == null)
      return this;
    else
      return venstreTre.finnMin();
  }

  public SubTre finnMaks() {
    if (hyreTre == null)
      return this;
    else
      return hyreTre.finnMaks();
  }

  public SubTre finnEtterflger() {
    if (hyreTre != null)
      return hyreTre.finnMin();

    if (forelder == null)
      return this;
    else if (forelder.finnVenstreTre() == this)
      return forelder;
    else
      return forelder.finnEtterflger();
  }

  public SubTre finnSubTre(int enVerdi) {
    if (verdi == enVerdi)
      return this;

    if (verdi > enVerdi) {
      if (venstreTre != null)
        return venstreTre.finnSubTre(enVerdi);
    }
    else {
      if (hyreTre != null)
        return hyreTre.finnSubTre(enVerdi);
    }

    return null;
  }

  public int finnVerdi() {
    return verdi;
  }

  /*
   * Sett-metoder
   */
  public void settVenstreTre(SubTre tre) {
    venstreTre = tre;
  }

  public void settHyreTre(SubTre tre) {
    hyreTre = tre;
  }

  public void settForelder(SubTre tre) {
    forelder = tre;
  }

  public void settVerdi(int nyVerdi) {
    verdi = nyVerdi;
  }

  /*
   * Metoden traverserer treet infiks, og returnerer en tekst med
   * innholdet, separert med blanktegn.
   */
  public String toString () {
    String returStreng = "";
    if (venstreTre != null) {
      returStreng = venstreTre.toString() + " ";
    }
    returStreng = returStreng + verdi;
    if (hyreTre != null) {
      returStreng = returStreng + " " + hyreTre.toString();
    }
    return returStreng;
  }

  /*
   * Metoden returnerer true dersom den oppgitte verdien finnes i
   * treet.
   */
  public boolean skEtterVerdi(int skeVerdi) {
    if (skeVerdi == verdi) return true;
    if (verdi > skeVerdi) {
      if (venstreTre != null) {
        return venstreTre.skEtterVerdi(skeVerdi);
      } else {
        return false;
      }
    } else {
      if (hyreTre != null) {
        return hyreTre.skEtterVerdi(skeVerdi);
      } else {
        return false;
      }
    }
  }
}

/* Utskriftvindu:
Treet: 1 4 5 6 8 10 13 14 15 16 17 21 22 23 25 26 27 28 31 35 36 39 40 44 49

Slettet tallet 28.
Treet: 1 4 5 6 8 10 13 14 15 16 17 21 22 23 25 26 27 31 35 36 39 40 44 49

Slettet tallet 36.
Treet: 1 4 5 6 8 10 13 14 15 16 17 21 22 23 25 26 27 31 35 39 40 44 49

Slettet tallet 35.
Treet: 1 4 5 6 8 10 13 14 15 16 17 21 22 23 25 26 27 31 39 40 44 49

Slettet tallet 16.
Treet: 1 4 5 6 8 10 13 14 15 17 21 22 23 25 26 27 31 39 40 44 49

Slettet tallet 5.
Treet: 1 4 6 8 10 13 14 15 17 21 22 23 25 26 27 31 39 40 44 49
*/