/*
 * BibliotekKlient2.java EL 2006-01-16
 *
 * Denne klienten brukes til  registrere utln, innlevering og reservering av bker.
 */
import java.io.*;
import java.rmi.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import static javax.swing.JOptionPane.*;

class Klient2GUI extends JFrame {
  private DefaultListModel listeInnhold = new DefaultListModel();
  private JList alleBker = new JList(listeInnhold);

  private JButton utlnKnapp = new JButton("Lne ut");
  private JButton reserverKnapp = new JButton("Reservere");
  private JButton kanselReservKnapp = new JButton("Kanselere reservasjon");
  private JButton visLntakerKnapp = new JButton("Vis lntakere");
  private JButton leverInnKnapp = new JButton("Levere inn");

  private JButton oppfriskKnapp = new JButton("Gjenoppfrisk data");

  private Bibliotek bibliotek;

  public Klient2GUI(String[] infoAlleBker, Bibliotek startTjener) {
    bibliotek = startTjener;
    setTitle("Registrering av utln, innlevering og reservering av bker");
    setLocation(200, 300);
    add(new JLabel("Alle bker i biblioteket"), BorderLayout.NORTH);
    for (String bokinfo : infoAlleBker) {
      listeInnhold.addElement(bokinfo);
    }
    alleBker.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    alleBker.setSelectedIndex(0);
    JScrollPane listeMedScrollbar = new JScrollPane(alleBker);
    add(listeMedScrollbar, BorderLayout.CENTER);
    add(new Knappepanel(), BorderLayout.SOUTH);
    pack();
  }

  private class Knappepanel extends JPanel {
    public Knappepanel() {
      add(utlnKnapp);
      add(reserverKnapp);
      add(kanselReservKnapp);
      add(visLntakerKnapp);
      add(leverInnKnapp);
      add(oppfriskKnapp);
      Knappelytter lytter = new Knappelytter();
      utlnKnapp.addActionListener(lytter);
      reserverKnapp.addActionListener(lytter);
      kanselReservKnapp.addActionListener(lytter);
      visLntakerKnapp.addActionListener(lytter);
      leverInnKnapp.addActionListener(lytter);
      oppfriskKnapp.addActionListener(lytter);
    }
  }

  private class Knappelytter implements ActionListener {
    public void actionPerformed(ActionEvent hendelse) {
      JButton kilde = (JButton) hendelse.getSource();
      int bokIndeks = alleBker.getSelectedIndex(); // til bruk ved oppdatering av listen
      String bokInfo = (String) alleBker.getSelectedValue();

      /* Bok-info'en er p n linje. Trenger ISBN som er frst p linjen */
      int pos = bokInfo.indexOf(Bok.sep);
      String isbn = bokInfo.substring(0, pos).trim();

      try {
        if (kilde == oppfriskKnapp) {
          String[] infoAlleBker = bibliotek.finnAlleBker();
          listeInnhold.clear();
          for (String bokinfo : infoAlleBker) {
            listeInnhold.addElement(bokinfo);
          }
          alleBker.setSelectedIndex(0);

        } else if (kilde == utlnKnapp) { // Lner ut et eksemplar av ei bok
          ArrayList<Integer> eksemplarer = bibliotek.finnEksInne(isbn);
          int eksNr = velgEksemplarnr(eksemplarer, "Eksemplarer til utln");
          if (eksNr > 0) {
            String navn = showInputDialog(null, "Navnet p kunden: ", "Utln", QUESTION_MESSAGE);
            if (navn != null && navn.trim().length() > 0) {
              bibliotek.lnUtEksemplar(isbn, navn, eksNr);
              showMessageDialog(null, "Utln av isbn  " + isbn + ", eksnr " + eksNr + " er registrert.", "Utln", INFORMATION_MESSAGE);
            } else showMessageDialog(null, "Utln er ikke registrert.", "Utln", INFORMATION_MESSAGE);
          }

        } else if (kilde == reserverKnapp) {  // Reserverer ei bok
          String navn = showInputDialog(null, "Navnet p kunden: ", "Reservasjon", QUESTION_MESSAGE);
          if (navn != null && navn.trim().length() > 0) {
            bibliotek.reserverEksemplar(isbn, navn);
            showMessageDialog(null, "Reservasjon av isbn  " + isbn + " er registrert.", "Reservasjon", INFORMATION_MESSAGE);
          } else showMessageDialog(null, "Reservasjon er ikke registrert.", "Reservasjon", INFORMATION_MESSAGE);

        } else if (kilde == kanselReservKnapp) { // Kanselerer reservasjon
          bibliotek.kanselerReservasjon(isbn);
          showMessageDialog(null, "Reservasjon av isbn  " + isbn + " er kanselert.", "Reservasjon", INFORMATION_MESSAGE);

        } else if (kilde == visLntakerKnapp) {
          ArrayList<String> lnere = bibliotek.finnLntakere(isbn);
          if (lnere.size() == 0) showMessageDialog(null, "Ingen lntakere.", "Lntakere", INFORMATION_MESSAGE);
          else {
            String liste = "Lntakerne:\n";
            for (String enLner : lnere) liste += enLner + "\n";
            showMessageDialog(null,  liste, "Lntakere", INFORMATION_MESSAGE);
            }
          } else {  // Returnerer ei bok
            ArrayList<Integer> eksemplarer = bibliotek.finnEksUte(isbn);
            int eksNr = velgEksemplarnr(eksemplarer, "Eksemplarer ute ");
            if (eksNr > 0) {
              bibliotek.returnerEksemplar(isbn, eksNr);
              showMessageDialog(null, "Retur av eksemplar nr " + eksNr + " er registrert.", "Retur av bok", INFORMATION_MESSAGE);
            } else showMessageDialog(null, "Retur av bok er ikke registrert.", "Retur av bok", INFORMATION_MESSAGE);
          }
          listeInnhold.set(bokIndeks, bibliotek.finnBokInfo(isbn));

      } catch (BiblException e) {
        showMessageDialog(null, e.getMessage(), "Melding", ERROR_MESSAGE);
        try { // oppdaterer raden dersom mulig
          listeInnhold.set(bokIndeks, bibliotek.finnBokInfo(isbn));
        } catch (Exception ex) {
        }
      } catch (RemoteException e) {
        showMessageDialog(null, "Kommunikasjonsproblem: " + e, "Melding", ERROR_MESSAGE);
        e.printStackTrace(); // til konsollvinduet
      }
    }

    private int velgEksemplarnr(ArrayList<Integer> eksemplarer, String tittel) {
      int valgtNr = 0;
      if (eksemplarer.size() > 0) {
        String[] valgmuligheter = new String[eksemplarer.size()];
        for (int i = 0; i < eksemplarer.size(); i++) {
          valgmuligheter[i] = "Eksemplar nr " + eksemplarer.get(i);
        }
        String valg = (String) showInputDialog(null, "Velg eksemplarnr", tittel, QUESTION_MESSAGE,
                                                null,  valgmuligheter, valgmuligheter[0]);
        if (valg  != null) {
          int pos = valg.lastIndexOf(' ');  // skal finne tallet som angir det valgte eksemplarnummeret
          String valgS = valg.substring(pos + 1);
          valgtNr = Integer.parseInt(valgS);  // skal aldri feile (!)
        }
      } else showMessageDialog(null, "Ingen eksemplarer", tittel, INFORMATION_MESSAGE);
      return valgtNr;
    }
  }
}

/* Klient for oppgave 19-3
class BibliotekKlient2 {
  public static void main(String[] args) throws Exception {
    Bibliotek tjener = (Bibliotek)Naming.lookup("rmi://localhost/Boktjener");
    String[] infoAlleBker = tjener.finnAlleBker();
    if (infoAlleBker.length == 0) {  // legger inn testdata dersom ingen data er lagt inn
      showMessageDialog(null, "Legger inn testdata\n", "Initiering", INFORMATION_MESSAGE);
      tjener.regNyBok(new Biblioteksbok("0-201-11743-6", "Turbo Pascal. A Problem Solving Approach",
                                                              "Elliot B. Koffamn"));
      tjener.regNyBok(new Biblioteksbok("0-13-089571-7", "C++ How to Program", "H. M. Deitel & P. J. Deitel"));
      tjener.regNyBok(new Biblioteksbok("0-13-081934-4", "Core Java Volume II - Advanced Features",
                                                              "Cay S. Horstmann, Gary Cornell"));
      tjener.regNyBok(new Biblioteksbok("0-596-00123-1", "Building Java Enterprise Applications",
                                                              "Brett McLaughlin"));
    }

    infoAlleBker = tjener.finnAlleBker();
    Klient2GUI gui = new Klient2GUI(infoAlleBker, tjener);
    gui.setVisible(true);
    gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }
} */

/* Klient for oppgave 20-1 */
class BibliotekKlient2 {
  public static void main(String[] args) throws Exception {
    DbWrapperFabrikk fabrikk = (DbWrapperFabrikk) Naming.lookup("rmi://localhost/Boktjener");
    Bibliotek tjener = fabrikk.lagDbWrapper();
    String[] infoAlleBker = tjener.finnAlleBker();
    Klient2GUI gui = new Klient2GUI(infoAlleBker, tjener);
    gui.setVisible(true);
    gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }
}