/**
 * Oppgave7_2.java - "Programmering i Java", 4.utgave - 2010-02-22
 *
 * Program for  holde orden p fravret i en skoleklasse.
 * Funksjonaliteten er lagt til klassen Skoleklasse.
 * Skoleelevene identifiseres med et nr. som gr fra 1 og oppover.
 * Antall elever kan ikke endres etter at Skoleklasse-objektet er opprettet.
 */

class Skoleklasse {
  // Fravret lagres i en heltallstabell. fravr[i] gir antall fravrsdager for elev i.
  private int[] fravr;

  public Skoleklasse(int antElever) {
    fravr = new int[antElever];
  }

  /**
   * Finner antall elever i klassen.
   */
  public int finnAntElever() {
    return fravr.length;
  }

  /**
   * Henter ut fravret til en bestemt elev.
   * Returnerer negativ verdi dersom ugyldig elevnr.
   */
  public int finnFravr(int elevNr) {
    int indeks = elevNr - 1;
    if (indeks >= 0 && indeks < fravr.length) {
      return fravr[indeks];
    } else {
      return -1;
    }
  }

  /**
   * Endrer fravret til en bestemt elev.
   * Returnerer det nye fravret, men negativ verdi dersom ugyldig elevnr (indeks).
   * Ingen kontroll av det tilfellet at fravret blir negativt.
   */
  public int endreFravr(int elevNr, int endring) {
    int indeks = elevNr - 1;
    if (indeks >= 0 && indeks < fravr.length) {
      fravr[indeks] += endring;
      return fravr[indeks];
    } else {
      return -1;
    }
  }

  /**
   * Finner gjennomsnittlig fravr.
   * Dersom antall elever er 0, returneres 0.
   */
  public double finnSnittFravr() {
    int sum = 0;
    for (int i = 0; i < fravr.length; i++) {
      sum += fravr[i];
    }
    if (fravr.length > 0) {
      return (double) sum / (double) fravr.length;
    } else {
      return 0.0;
    }
  }

  /**
   * Finner antall elever uten fravr.
   */
  public int finnAntEleverUtenFravr() {
    int antall = 0;
    for (int i = 0; i < fravr.length; i++) {
      if (fravr[i] == 0) {
        antall++;
      }
    }
    return antall;
  }
}

/**
 * Testklient iht. oppgaveteksten.
 */
class Oppgave7_2 {
  public static void main(String[] args) {
      final double TOLERANSE = 0.00001; // for kontroll av beregnede gjennomsnittsverdier
      System.out.println("Antall tester: 5");

      // Testdatasett 1
      Skoleklasse klasse1 = new Skoleklasse(0);
      if (klasse1.finnAntElever() == 0 &&
          klasse1.finnFravr(1) == -1 &&
          (Math.abs(klasse1.finnSnittFravr()) < TOLERANSE) &&
          klasse1.finnAntEleverUtenFravr() == 0 &&
          klasse1.endreFravr(7, 3) == -1) {
        System.out.println("Skoleklasse: Test 1 vellykket");
      }

      // Testdatasett 2
      Skoleklasse klasse2 = new Skoleklasse(1);
      if (klasse2.endreFravr(1, 5) == 5 &&
          klasse2.finnAntElever() == 1 &&
          klasse2.finnFravr(1) == 5 &&
          (Math.abs(klasse2.finnSnittFravr() - 5) < TOLERANSE) &&
          klasse2.finnAntEleverUtenFravr() == 0 &&
          klasse2.endreFravr(7, 3) == -1) {
        System.out.println("Skoleklasse: Test 2 vellykket");
      }

      // Testdatasett 3
      Skoleklasse klasse3 = new Skoleklasse(5);
      if (klasse3.finnAntElever() == 5 &&
          klasse3.finnFravr(1) == 0 &&
          (Math.abs(klasse3.finnSnittFravr()) < TOLERANSE) &&
          klasse3.finnAntEleverUtenFravr() == 5 &&
          klasse3.endreFravr(7, 3) == -1) {
        System.out.println("Skoleklasse: Test 3 vellykket");
      }

      // Testdatasett 4, fortsetter med klasse3
      klasse3.endreFravr(1, 5);
      klasse3.endreFravr(3, 2);
      klasse3.endreFravr(5, 1);
      if (klasse3.finnAntElever() == 5 &&
          klasse3.finnFravr(1) == 5 &&
          (Math.abs(klasse3.finnSnittFravr() - 1.6) < TOLERANSE) &&
          klasse3.finnAntEleverUtenFravr() == 2 &&
          klasse3.endreFravr(7, 3) == -1) {
        System.out.println("Skoleklasse: Test 4 vellykket");
      }

      // Testdatasett 5, fortsetter med klasse 3
      klasse3.endreFravr(1, 2);
      klasse3.endreFravr(2, 1);
      klasse3.endreFravr(3, 4);
      klasse3.endreFravr(4, 2);
      klasse3.endreFravr(5, 1);
      if (klasse3.finnAntElever() == 5 &&
          klasse3.finnFravr(1) == 7 &&
          (Math.abs(klasse3.finnSnittFravr() - 3.6) < TOLERANSE) &&
          klasse3.finnAntEleverUtenFravr() == 0 &&
          klasse3.endreFravr(7, 3) == -1) {
        System.out.println("Skoleklasse: Test 5 vellykket");
      }
  }
}
